
/*****************************************************************************/
/*                                                                           */
/* Directives d'execution (du type #define en C)                             */
/*                                                                           */
/*****************************************************************************/

:-module(define,
	[
		define/1,
		redefine/1,
		undefine/1,
		ifdef/1,
		ifdef/2
	]
).

/*****************************************************************************/
/*                                                                           */
/* Definition d'un symbole                                                   */
/*                                                                           */
/*****************************************************************************/

define((A,B)):-
	!,
	define(A),
	define(B).

define(A=X):-
	!,
	(	ifdef(A)
	->	warning("%t redefined.\n",[A]),
		undefine(A)
	;	true
	),
	recordz(A,X).

define(A):-
	define(A=true).

/*****************************************************************************/
/*                                                                           */
/* Redefinition d'un symbole                                                 */
/*                                                                           */
/*****************************************************************************/

redefine(A):-
	undefine(A),
	define(A).

/*****************************************************************************/
/*                                                                           */
/* Test de presence d'un symbole                                             */
/*                                                                           */
/*****************************************************************************/

ifdef(A=X):-
	!,
	recorded(A,X).

ifdef(A):-
	recorded(A,_).

ifdef(COND,GOAL):-
	(	ifdef(COND)
	->	GOAL
	;	true
	).

/*****************************************************************************/
/*                                                                           */
/* Suppression d'un symbole                                                  */
/*                                                                           */
/*****************************************************************************/

undefine((A,B)):-
	!,
	undefine(A),
	undefine(B).

undefine(A):-
	recorded(A,_,REF),
	erase(REF),
	fail.
undefine(A=X):-
	recorded(A,X,REF),
	erase(REF),
	fail.
undefine(_).
