
/*****************************************************************************/
/*                                                                           */
/* Un peu de tout                                                            */
/*                                                                           */
/*****************************************************************************/

:-module(tools,
	[
		bracket_to_list/2,

		member/3,

		error/1,
		error/2,
		warning/1,
		warning/2,
		chk/2,
		chk/3,

		empty/1,

		empty_module/1,

%		subst/3,

		in/2,

		menu/3,
		readln/1,

		reinit/0,

		gen_ref/1,

		max/2,

		merge_sort/2,

		for/4,

		rasserta/1,
		rassertz/1,
		rretracta/1, 
		rretractz/1,

		depth/2

	]
).

:-use_module([decls,unify]).


/*****************************************************************************/
/*                                                                           */
/* Conversion {} en []                                                       */
/*                                                                           */
/*****************************************************************************/

bracket_to_list(X,X):-
	var(X),
	!.
bracket_to_list(X,Y):-
	X=..[F|ARGS],
	maplist(bracket_to_list,ARGS,LARGS),
	(	F='{}'
	->	to_list(LARGS,Y)
	;	Y=..[F|LARGS]
	).

to_list(X,[X]):-
	var(X),
	!.
to_list([A],L):-
	!,
	to_list(A,L).
to_list((A,B),[A|LB]):-
	!,
	to_list(B,LB).
to_list(X,[X]).


/*****************************************************************************/
/*                                                                           */
/* Member + extraction                                                       */
/*                                                                           */
/*****************************************************************************/

member(E,[E|L],L).
member(E,[F|L1],[F|L2]):-member(E,L1,L2).


/*****************************************************************************/
/*                                                                           */
/* in : parcours d'expression                                                */
/*                                                                           */
/*****************************************************************************/

X in X :- var(X),!.

X in X and _.
X in _ and Y :- X in Y.

X in X or _.
X in _ or Y :- X in Y.

X in X :- X\=(_ or _), X\=(_ and _), X\='.'(_,_).

/*****************************************************************************/
/*                                                                           */
/* Erreurs                                                                   */
/*                                                                           */
/*****************************************************************************/

error(MSG):-error(MSG,[]).
error(MSG,ARGS):-
	writef("ERROR : "), writef(MSG,ARGS),
	abort.

warning(MSG):-warning(MSG,[]).
warning(MSG,ARGS):-
	writef("WARNING : "), writef(MSG,ARGS).

:-module_transparent chk/2, chk/3.

chk(COND,MSG):-chk(COND,MSG,[]).
chk(COND,MSG,ARGS):-
	(	COND
	;	error(MSG,ARGS)
	),
	!.


/*****************************************************************************/
/*                                                                           */
/* Remise a zero d'une entree dans la BD                                     */
/*                                                                           */
/*****************************************************************************/

empty([]):-!.
empty([K|KS]):-!,empty(K),empty(KS).
empty(K):-forall(recorded(K,_,REF),erase(REF)).


/*****************************************************************************/
/*                                                                           */
/* Remise a zero d'un module                                                 */
/*                                                                           */
/*****************************************************************************/

empty_module([]):-!.
empty_module([M|MS]):-!,empty_module(M),empty_module(MS).
empty_module(M):-forall(clause(M:_,_,REF),erase(REF)).


/*****************************************************************************/
/*                                                                           */
/* Substitutions dans un terme                                               */
/*                                                                           */
/*****************************************************************************/

/* Dans certain cas, on substitue des termes qui partagent des variables
 * Il faut utiliser l'unification avec verification des occurences
 * (sinon bug dans la regle X b Y b P => -(X b Y b -P) au moment de la
 * compilation des regles de mise a jour
 */

/* Cette version ne marche pas, il faut utiliser celle de subst.pl

subst(X,_,X):-var(X),!.
subst([],_,[]):-!.
subst([T|R],S,[TS|RS]):-
	!,
	subst(T,S,TS),
	subst(R,S,RS).

subst(T,S,TS):-
	T=..[F|A],
	(	A=[]		% T est un atome
	->	(	(	clean_memberchk(TS1/T,S)			% T est un atome substituable
			;	clean_memberchk(TS1/T#COND,S),
				COND
			),
			TS1\=T,
			subst(TS1,S,TS)
		;	TS=T
		)
	;	subst(A,S,AS),
		TS0=..[F|AS],
		(	(	clean_memberchk(TS1/TS0,S)		% T est un terme substituable
			;	clean_memberchk(TS1/TS0#COND,S),
				COND
			),
			TS1\=TS0,
			subst(TS1,S,TS)
		;	TS=TS0
		)
	),
	!.
*/

/*****************************************************************************/
/*                                                                           */
/* memberchk avec occur check                                                */
/*                                                                           */
/*****************************************************************************/

clean_memberchk(X,[Y|L]):-
	copy_term(Y,Z),					% il ne faut pas modifier la liste
	(	X*=Z
	;	clean_memberchk(X,L)
	),
	!.


/*****************************************************************************/
/*                                                                           */
/* Menu textuel                                                              */
/*                                                                           */
/*****************************************************************************/

:-module_transparent menu/3.

menu(TITRE,OPTIONS,PROMPT):-
	writef("\n%s\n\n",[TITRE]),
	forall(member(CH:OPT:_,OPTIONS),
		(	writef("%s : ",[CH]),
			(	OPT=(FORMAT,ARGS)
			->	writef(FORMAT,ARGS)
			;	writef(OPT)
			),
			nl
		)
	),
	writef("\n%s",[PROMPT]),
	repeat,
		get_single_char(C),
	memberchk([C]:_:BUT,OPTIONS),
	writef("%s\n",[[C]]),
	(	BUT
	;	writef("Erreur : La commande a echoue\n")
	),
	!.

readln(LN):-
	readln([],LN),
	!.

readln(LN0,LN):-
	get_single_char(CH),
	(	CH==8	->	(	LN0=[_|LN1], writef([8,32,8])
					;	LN1=[]
					),
					readln(LN1,LN)
	;	CH>=32	->	writef("%s",[[CH]]),
					readln([CH|LN0],LN)
	;	(	CH==13
		;	CH==10
		)		->	reverse(LN0,LIST),
					name(LN,LIST)
	;	readln(LN0,LN)
	).


/*****************************************************************************/
/*                                                                           */
/* Reinitialisation de tous les modules                                      */
/*                                                                           */
/*****************************************************************************/

:-module_transparent 
	reinit/0,
	reinit/1.

/* Pour etre reinitialisable, un module doit avoir un predicat init/0
 */

reinit:-
	findall(M:INIT,
		(	current_module(M),
			clause(M:init,INIT)
		),
		INITS),
	reinit(INITS).

reinit([]).
reinit([INIT|INITS]):-
	(	memberchk(INIT,INITS)
	;	INIT
	),
	!,
	reinit(INITS).


/*****************************************************************************/
/*                                                                           */
/* Generateur de references                                                  */
/*                                                                           */
/*****************************************************************************/

init:-flag(gen_ref,_,0).

gen_ref(R):-flag(gen_ref,R,R+1).


/*****************************************************************************/
/*                                                                           */
/* Max d'une liste                                                           */
/*                                                                           */
/*****************************************************************************/

max([],0):-!.

max([X|L],M):-
	max(L,Y),
	M is max(X,Y).



/*****************************************************************************/
/*                                                                           */
/* Tri decroissant selon une cle                                             */
/*                                                                           */
/*****************************************************************************/

merge_sort(L,S):-
	length(L,N),
	merge__go(N,L,S,[]).

merge__go(0,L,[],L):-!.
merge__go(1,[X|L],[X],L):-!.
merge__go(N,L,S,R):-
	N1 is N>>1,		merge__go(N1,L,S1,R1),
	N2 is N-N1,		merge__go(N2,R1,S2,R),
	merge_2(S2,S1,S).

merge_2([],S,S):-!.
merge_2([KX:X|L1],[KY:Y|L2],[KX:X|L]):-
	KY<KX,
	!,
	merge_2(L1,[KY:Y|L2],L).
merge_2(L1,[KY:Y|L2],[KY:Y|L]):-
	merge_2(L2,L1,L).

/*****************************************************************************/
/*                                                                           */
/* Boucle for                                                                */
/*                                                                           */
/*****************************************************************************/

:-module_transparent for/4.

for(I,MIN,MAX,BODY):-
	VMIN is MIN,
	VMAX is MAX,
	between(VMIN,VMAX,I),
		call(BODY),
	fail.
for(_,_,_,_).


/*****************************************************************************/
/*                                                                           */
/* assert et retract reversibles                                             */
/*                                                                           */
/*****************************************************************************/

:-module_transparent rasserta/1, rassertz/1, rretracta/1, rretractz/2.

rasserta(X):-
	asserta(X,REF),
	(	true
	;	erase(REF), fail
	).

rassertz(X):-
	assertz(X,REF),
	(	true
	;	erase(REF), fail
	).

rretracta(X):-
	retract(X),
	(	true
	;	asserta(X),
		fail
	).

rretractz(X):-
	retract(X),
	(	true
	;	assertz(X),
		fail
	).

/*****************************************************************************/
/*                                                                           */
/* Depth                                                                     */
/*                                                                           */
/*****************************************************************************/

depth(T,D):-
	T=..[_|A],
	maplist(depth,A,DA),
	max(DA,M),
	D is M+1,
	!.
depth(_,0).
